@@ -242,6 +242,19 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
         The prepare_[master|slave] functions will put the instance in a state whereby
         '/etc/init.d/postgresql start' can be executed.
     """
+    def _get_conn(self, host=None, dbname=None, user=None):
+        """ Returns a connection to postgresql server.
+        """
+        conn_str = ''
+        if host:
+            conn_str += 'host=%s ' % host
+        if dbname:
+            conn_str += 'dbname=%s ' % dbname
+        if user:
+            conn_str += 'user=%s ' % user
+
+        return psycopg2.connect(conn_str)
+
     def start_process(self):
         """ Starts postgresql using the init.d scripts.
         """
@@ -250,16 +263,16 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
     def poll_process(self):
         pass
 
-    def write_recovery_conf(self):
+    def write_recovery_conf(self, template_path):
         """ Using the template specified in settings, create a recovery.conf file in the
             postgres config dir.
         """
-        self.logger.info('Writing recovery file using template %s' % settings.RECOVERY_TEMPLATE)
+        self.logger.info('Writing recovery file using template %s' % template_path)
         data = dict(self.metadata.items() + self.settings.items())
         data.update(
             {'master_cname': self.master_cname}
         )
-        template_file = open(settings.RECOVERY_TEMPLATE, 'r')
+        template_file = open(template_path, 'r')
         template = template_file.read()
         template_file.close()
         output = open(settings.RECOVERY_FILENAME, 'w')
@@ -292,30 +305,19 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
         """
         # TODO remove acquire call, this happens after proc is started
         self.acquire_master_cname()
+        self.write_recovery_conf(settings.RECOVERY_TEMPLATE_MASTER)
         self.configure_cron_backup()
         # TODO apply some tags here to show the role of the instance
 
     def prepare_slave(self):
         """ Init postgres as a read-slave by writing a recovery.conf file.
         """
-        self.write_recovery_conf()
+        self.write_recovery_conf(settings.RECOVERY_TEMPLATE_SLAVE)
         self.logger.info('Instance configured as a slave')
         # TODO remove cname call, this happens after proc is started
         self.add_to_slave_cname_pool()
         # TODO apply some tags here to show the role of the instance
 
-    def _get_conn(self, host=None, dbname=None, user=None):
-        """ Returns a connection to postgresql server.
-        """
-        conn_str = ''
-        if host:
-            conn_str += 'host=%s ' % host
-        if dbname:
-            conn_str += 'dbname=%s ' % dbname
-        if user:
-            conn_str += 'user=%s ' % user
-
-        return psycopg2.connect(conn_str)
 
     def check_master(self):
         """ Returns true if there is a postgresql server running on the master CNAME
@@ -324,7 +326,13 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
             master in the cluster.
         """
         # TODO untested
-        conn = self._get_conn(host=self.master_cname)
+        self.logger.info('Checking master DB at %s' % self.master_cname)
+        try:
+            conn = self._get_conn(host=self.master_cname)
+        except psycopg2.OperationalError:
+            self.logger.info('Connecting to master failed')
+            return False
+
         cur = conn.cursor()
         cur.execute('SELECT pg_is_in_recovery()')
         res = cur.fetchone()
@@ -346,6 +354,7 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
             the server is in recovery mode (i.e. it is a read slave).
         """
         # TODO untested
+        self.logger.info('Checking slave DB on localhost')
         conn = self._get_conn()
         cur = conn.cursor()
         cur.execute('SELECT pg_is_in_recovery()')
@@ -358,7 +367,12 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
 
             If force is True, safety checks are ignored and the promotion is forced.
         """
-        active_master = self.check_master()
+        try:
+            active_master = self.check_master()
+        except psycopg2.OperationalError, e:
+            print 'Could not connect to master'
+            active_master = False
+
         if active_master == True:
             print 'There is an active server at %s' % self.master_cname
             if force == False:
@@ -379,6 +393,7 @@ class PostgresqlCluster(EC2Mixin, BaseCluster):
                 # TODO custom exception?
                 raise Exception(e.output)
             else:
+                print e.output
                 raise e
 
         # If we get here, then postgresql should have been successfully promoted.
